from django.http import Http404, HttpResponse
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.db import IntegrityError, DatabaseError
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status, routers
from datetime import datetime
from django.utils.timezone import now
from django.http import HttpResponse
from django.conf import settings
from datetime import datetime
import json

class ProcessPatient(APIView):

    def post(self, request, format=None):
        # If this patient does not exist, we create the patient and guardian
        patient = request.DATA
        data = { 'hide' : True }
        if(is_user_or_site_live(patient)):
            data = runScript(patient)

        # Hide if BMI is missing, non-numeric, or less than 85%ile
        try:
            data['hide'] = (float(data['bmi']['value']) < 85.0)
        except (KeyError, ValueError):
            data['hide'] = True

        return Response(data)

#functions
# lookup the value of a particular data-point in the patient data
def lookupValueScrape(scrape,link,identifier):
    if(identifier in scrape[link]):
        return scrape[link][identifier]
    else:
        return ""
# breakout the patient identifiers into a single associative array/dictionary
def fixIdentifiers(scrape):
    identifiers={}
    for identifier in scrape['identifiers']:
        identifiers[identifier['type']]=identifier['id']
    return identifiers
#index o=lookup using generated key
def indexLookup(index,key,default):
    if key in index:
        return index[key]
    return default
# method does nothing yet
def xmlDDIndex(xml,valueId,value,subConceptValueID,subConceptValue):
    return
#method converts the three variables to the reveseXML lookup key
def generateKey(v1,v2,v3=None):
    if v3 is None:
        return v1+"_"+v2
    return v1+"_"+v2+"_"+v3
#convert from birthday to age
def convertYears(bd):
  #source: http://www.romcartridge.com/2010/01/javascript-function-to-calculate-age.html, modified
  #birthday (yyyy-mm-dd)
  bdSplits=bd.split("-")
  birthYear=int(bdSplits[0])
  birthMonth=int(bdSplits[1])-1
  birthDay=int(bdSplits[2])
  todayDate = str(datetime.now())
  todayYear = datetime.now().year
  todayMonth = datetime.now().month
  todayDay = datetime.now().day
  #age calculator
  age = todayYear - birthYear
  if (todayMonth < birthMonth):
        age=age-1
  if (birthMonth == todayMonth):
        if(todayDay < birthDay):
            age=age-1
  return age;
#creates reverseLookup table for cpt/floid+value[+subConceptName]
def reverseXML(concepts):
    reverse={}
    #for each concept
    for concept in concepts:
        valId=concept['valueId']
        #for each value
        if isinstance(concept['value'],str):
            val = concept['value']
            # for each subconcept
            if "subConceptValue" in concept:
                for subconcept in concept['subConceptValue']:
                        reverse[valId+"_"+val+"_"+subconcept]=concept['conceptId']
            else:
                reverse[valId+"_"+val]=concept['conceptId']
        else:
            for value in concept['value']:
                val = value
                # for each subconcept
                if "subConceptValue" in concept:
                    for subconcept in concept['subConceptValue']:
                            reverse[valId+"_"+val+"_"+subconcept]=concept['conceptId']
                else:
                    reverse[valId+"_"+val]=concept['conceptId']
    return reverse
def dateInt(datestring):
    return int(datestring[0:4]+datestring[5:7]+datestring[8:10])
#returns a flat of all the resulted values and 
def processPatient(patient,index):
    labs={}
    for instance in patient['encounters']:
        instant = instance.get('instant',"0000000000")# time of the encounter TODO test this works
        instantint = dateInt(instant)# int of time stamp
        for proc in instance['procedures']:
            if "results" in proc:
                for result in proc['results']:
                    #create a lookup key from the lab data
                    local_key=indexLookup(index,generateKey("cpt",proc['cpt'],result['component']),'OOI_Error')
                    if local_key in labs:
                        #if the value is later - replace
                        if(instantint>labs[local_key]['instantint']):
                            #assign the results
                            if "numericResult" in result:
                                value=result["numericResult"]
                            else:
                                value=result['textResult']
                            #insert the object
                            labs[local_key]={
                                "instantint":dateInt(instant),
                                "instant":instant,
                                "value":value,
                                "low":result.get('referenceLow'),
                                "high":result.get('referenceHigh')                    
                            }
                    #local_key not yet defined
                    else:
                       #assign the results
                        if "numericResult" in result:
                            value=result["numericResult"]
                        else:
                            value=result.get('textResult',"")#TODO test this works
                        #insert the object
                        labs[local_key]={
                            "instantint":dateInt(instant),
                            "instant":instant,
                            "value":value,
                            "low":result.get('referenceLow'),
                            "high":result.get('referenceHigh')
                        }
        for flow in instance['flowsheets']:
            local_key=indexLookup(index,generateKey("floid",flow['floid']),'OOI_Error')
            if local_key in labs:
                if(instantint>labs[local_key]['instantint']):               
                    labs[local_key]={
                        "instantint":dateInt(instant),
                        "instant":instant,
                        "value":flow['value']
                    }
            else:
                labs[local_key]={
                    "instantint":dateInt(instant),
                    "instant":instant,
                    "value":flow['value']
              }
    #data2 = sorted(data,key=lambda k: k['instantint'])
    return labs

# TODO: remove this function after pilot phase
# HWI PILOTS
# Jeremy Michel   MICHELJ
# Bob Grundmeier 1002
def is_user_or_site_live(process_data_payload):
    # phased rollout by department
    live_dates = { '51606012'  : '2015-02-01',  # Cobbs Creek      51606012 
                   '69318012'  : '2015-04-01',  # Roxborough       69318012 
                   '101018001' : '2015-08-05',  # Karabots        101018001 
                   '50601012'  : '2015-05-13',  # S Philly         50601012 
                   '89296012'  : '2015-05-13',  # Faculty practice 89296012 
                   '93312012'  : '2016-08-05',  # Highpoint        93312012 
                   '10700012'  : '2016-08-05' } # Flourtown        10700012 
                   
                   #Chestnut Hill
    dept_id = get_id(process_data_payload, 'DEPARTMENT_ID')
    go_live = live_dates.get(dept_id, '2016-08-05') # live for all registered sites as of 2015-06-01

    # determine date of current contact -- if it is on or after go_live date then return true
    # exploit natural ordering of ISO format date strings to compare dates
    enc_id = get_id(process_data_payload, 'ENCOUNTER_ID')
    for encounter in process_data_payload.get('encounters',[]):
        if enc_id == get_id(encounter, 'ENCOUNTER_ID') and encounter['instant'] >= go_live:
            return True

    # fallback, check to see if current user is a pilot user. Return true if user id is in the list
    return (get_id(process_data_payload, 'USER_ID') or '').strip() in ['MICHELJ']
# TODO: update offsite list - may need to do third category for NJ practices
def departmentLookup(department):
    onsite = ['51606012','101018001','50601012','89296012']
    offsite = ['69318012','offsite','93312012','10700012']

    if (department in onsite):
        return "onsite"
    if (department in offsite):
        return "offsite"
    return "offsite"

#quick script to modify the payor/lab based on the department onsite/offsite drawing ()
def associatePayorDepartment(payor_class,department):
    #base case - no modify
    if(payor_class in ['chop','quest','labcorp']):
        return payor_class
    if(payor_class == 'chop-quest'):
        if(department == 'onsite'):
            return 'chop'
        return 'quest'
    if(payor_class == 'chop-labcorp'):
        if(department == 'onsite'):
            return 'chop'
        return 'labcorp'
    return 'chop'

def get_id(obj, idval):
    if(obj is None or obj == {}):
        return ""
    for identifier in obj.get("identifiers"):
        if(identifier["type"] == idval):
            return identifier["id"]
    return None
#data
def runScript(patient):
    xml = {"concept":[{"conceptId":"bmi", "conceptType":"flowsheets", "valueId":"floid", "value":"73008"}, {"conceptId":"sbptile", "conceptType":"flowsheets", "valueId":"floid", "value":"1004"}, {"conceptId":"dbptile", "conceptType":"flowsheets", "valueId":"floid", "value":"1005"}, {"conceptId":"LDL", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["CHOLESTEROL, DIRECT LDL", "LDL CHOL. (DIRECT)", "LDL CHOLESTEROL CALC", "LDL CHOLESTEROL", "LDL-C", "LDL-CHOL", "LDL-CHOLESTEROL", "LDL-P", "LDL-R (REAL)-C"], "value":["302756.5","90515.001", "303766.500", "306756.500", "7600.001", "316655.500", "303756.500", "235011.500", "14852.001", "302058.501", "302805.500", "235010.500", "033969.500", "033886.500", "221010.500", "80061", "80061.002", "320301.500", "303085.500", "8293.001", "120295.500"] }, {"conceptId":"totalChol", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["CHOLESTEROL, SERUM", "CHOLESTEROL, TOTAL", "CHOLESTEROL", "CHOLESTEROL-NUTRITION", "SUM TOTAL CHOLESTEROL","CHOLESTEROL, TOTAL POC"], "value":["302756.5","82465.996","90515.001", "303766.500", "306756.500", "7600.001", "316655.500", "303756.500", "235011.500", "14852.001", "302058.501", "302805.500", "235010.500", "033969.500", "033886.500", "221010.500", "80061", "80061.002", "320301.500", "303085.500", "82465.002", "001065.500", "334.001"] }, {"conceptId":"HDL", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["HDL CHOLESTEROL", "HDL-2 (MOST PROTECTIVE)", "HDL-3 (LESS PROTECTIVE)", "HDL-C", "HDL-P (TOTAL)","HDL, POC"], "value":["302756.5","82465.996","90515.001", "303766.500", "306756.500", "7600.001", "316655.500", "303756.500", "235011.500", "14852.001", "302058.501", "302805.500", "235010.500", "033969.500", "033886.500", "221010.500", "80061", "80061.002", "320301.500", "303085.500", "001925.500", "80061.001", "608.001"] }, {"conceptId":"tg", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":"TRIGLYCERIDES", "value":["302756.5","90515.001", "303766.500", "306756.500", "7600.001", "316655.500", "303756.500", "235011.500", "14852.001", "302058.501", "302805.500", "235010.500", "033969.500", "033886.500", "221010.500", "80061", "80061.002", "320301.500", "303085.500", "896.001", "001172.500"] }, {"conceptId":"nonHDL", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["NON HDL CHOLESTEROL", "NON-HDL CHOLESTEROL"], "value":["90515.001", "303766.500", "306756.500", "7600.001", "316655.500", "303756.500", "235011.500", "14852.001", "302058.501", "302805.500", "235010.500", "033969.500", "033886.500", "221010.500", "80061", "80061.002", "320301.500", "303085.500"] }, {"conceptId":"ast", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["AST", "AST (SGOT)", "ASPARTATE AMINOTRANSFERASE", "ASPARTATE AMINOTRANSFERASE", "SGOT"], "value":["822.001", "84450.999", "80076.999", "10231.001", "001123.500", "10256.001", "322000.500", "302085.500", "80053.003"] }, {"conceptId":"alt", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["ALANINE AMINOTRANSFERASE", "ALANINE AMINOTRANSFERASE NO RF", "ALT", "ALT (SGPT)", "SGPT"], "value":["302756.5","84460.999", "80076.999", "823.001", "10231.001", "84460.001", "001545.500", "10256.001", "322000.500", "302085.500", "80053.003"] }, {"conceptId":"glucose", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["GLUCOSE", "GLUCOSE, SERUM", "GLUC","GLUCOSE, RANDOM","GLUCOSE, FASTING","GLUCOSE, POINT-OF-CARE"], "value":["1302.5","348014.5","82947.997","82962.999", "80048.999", "82947.002", "82947.003", "10165.001", "10231.001", "484.001", "483.001", "001818.500", "001032.500", "484.002", "82948.998", "82948.015", "82947.005", "82962.998", "322000.500", "302085.500", "80053.003"] }, {"conceptId":"hgba1c", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["HEMOGLOBIN A1C", "A1C", "HEMOGLOBIN A1C-POINT OF CARE", "ACCU-CHEK A1C", "HEMOGLOBIN A1C (FINGERSTICK)"], "value":["91314.500", "83036.001", "83036.003", "8181.001", "501963.500", "501963.500", "102525.500", "001453.500", "496.001", "83036.002", "001172.500", "501270.500"] }, {"conceptId":"vitD", "conceptType":"procedure", "valueId":"cpt", "subConcept":"components", "subConceptValueId":"name", "subConceptValue":["VITAMIN D, 25-HYDROXY,SERUM", "VITAMIN D, 25-HYDROXY", "VITAMIN D, 25-OH, TOTAL", "VITAMIN D, 25-OH, D3", "VITAMIN D, 25-OH, D2", "VITAMIN D 25OH", "VITAMIN D, 25-HYDROXY, SERUM"], "value":["5678.001", "82652.001", "080325.500", "82306.001", "AMD17306.001", "081950.500", "17306.001", "500510.500", "91935.001", "16761.001"] }, {"conceptId":"osa", "conceptType":"diagnosis", "valueId":"icd", "value":"786.09"}, {"conceptId":"pseudotumor", "conceptType":"diagnosis", "valueId":"icd", "value":"PSEUDOTUMOR"}, {"conceptId":"migraines", "conceptType":"diagnosis", "valueId":"icd", "value":"MIGRAINES"}, {"conceptId":"blount", "conceptType":"diagnosis", "valueId":"icd", "value":"BLOUNTS"}, {"conceptId":"isOverweight", "conceptType":"problem", "valueId":"icd", "value":"15"}, {"conceptId":"isDiabetic", "conceptType":"problem", "valueId":"icd", "value":"15"}, {"conceptId":"fhDiabetes", "conceptType":"FH", "valueId":"floid", "value":"diabetes"}, {"conceptId":"fhObesity", "conceptType":"FH", "valueId":"floid", "value":"obesity"}, {"conceptId":"fhHTN", "conceptType":"FH", "valueId":"floid", "value":"htn"}, {"conceptId":"fhHyperlipid", "conceptType":"FH", "valueId":"floid", "value":"hyperlipid"}, {"conceptId":"chop-quest", "conceptType":"demographics", "valueId":"payor", "value":["MEDICAID", "Other", "CIGNA", "Coventry Cares - Medicaid", "United Healthcare Community Plans (UHCCP)", "United Healthcare (UHC)", "TRICARE", "Coventry Healthcare"] }, {"conceptId":"chop-labcorp", "conceptType":"demographics", "valueId":"payor", "value":["KEYSTONE FIRST MEDICAID HMO", "Keystone First - Medicaid"] }, {"conceptId":"quest", "conceptType":"demographics", "valueId":"payor", "value":["AETNA", "Qualcare", "Aetna Better Health", "AETNA BETTER HEALTH", "AETNA BETTER HEALTH KIDS", "AETNA EPO", "AETNA HMO", "AETNA INDEMNITY", "AETNA POS", "AETNA PPO"] }, {"conceptId":"labcorp", "conceptType":"demographics", "valueId":"payor", "value":["Keystone Health Plan East", "Keystone HP East POS", "KHPE", "Personal Choice", "PERSONAL CHOICE PPO", "KEYSTONE HP EAST HMO", "KEYSTONE HP EAST POS", "Independence Blue Cross (IBC)", "Keystone First Health Plan", "Personal Choice (IBC)", "Highmark Blue Cross Blue Shield of Delaware", "Horizon BCBSNJ","Amerihealth"] }, {"conceptId":"male", "conceptType":"demographics", "valueId":"sex", "value":["Male", "M"] }, {"conceptId":"female", "conceptType":"demographics", "valueId":"sex", "value":["Female", "F"] } ] }
    patient_data = {}
    #build
    # FROM THE PATIENT SCRAPE
    # observations have a single .value and .datetime (flowsheets and floid are the access key)
    # --> bmi, sbptile, dpbtile - spot created
    # lab values have a value, low, high and datetime (from the patient scrape)  (procedures, )
    #   return {"value":value,"low":low,"high":high,"datetime":instant};
    # ---> LDL, HDL, tg, totalChol, nonHDL, vitD, glucose, hgba1c, ast, alt - spot created
    # other 'results'
    # --> sleepstudy (.results) - empty string
    #conditions (as spelled here) (.status)
    # --> diabetes, snoring, pseudotumor, migrains, acne, acantosis, blount, jointpain - false currently
    #
    #pre-process the XML/JSON object
    xml_hash = reverseXML(xml['concept'])
    #grab the free patient data
    bd = patient['birthDate']
    name = lookupValueScrape(patient,"demographics","name")
    street = lookupValueScrape(patient,"demographics","street")
    city = lookupValueScrape(patient,"demographics","city")
    zipCode = lookupValueScrape(patient,"demographics","zipcode")
    gender = lookupValueScrape(patient,"demographics","sex")
    identifiers=fixIdentifiers(patient)

    #handle payor detals
    payor = identifiers.get('PAYOR','Other')
    #require a payor
    if(payor is ''):
        payor="Other"
    payor_class = indexLookup(xml_hash,generateKey("payor", payor),"chop")

    department = get_id(patient, 'DEPARTMENT_ID')
    #require a department
    if(department is ''):
        department="offsite"
    department_class = departmentLookup(department)

    #correct the payor class for the department
    payor_class = associatePayorDepartment(payor_class,department_class)
    
    #convert from free data to the patient_data
    patient_data['constants'] = {
        "age_years":convertYears(bd),
        "datetime":"%datetime",
        "name":name,
        "first_name":name[name.index(",")+1:],
        "last_name":name.split(",")[0],
        "address":street + "\n" + city + " " + zipCode,
        "street":street,
        "city":city,
        "payor":payor_class,
        "department":department,
        "department_class":department_class,
        "gender":indexLookup(xml_hash,generateKey("sex", gender),"unknown"),
        "zip":zipCode
    }    
    #process the patient
    patient_simple = processPatient(patient,xml_hash)
    #index the processed data into patient_data
    for (i, item) in enumerate(patient_simple):
        patient_data[item]=patient_simple[item]
    #todo - replace this in future with ICD-9 checking
    patient_data['sleepstudy']={"results":""}
    patient_data['diabetes']={"status":"false"}
    patient_data['snoring']={"status":"false"}
    patient_data['pseudotumor']={"status":"false"}
    patient_data['migraines']={"status":"false"}
    patient_data['acne']={"status":"false"}
    patient_data['acanthosis']={"status":"false"}
    patient_data['blount']={"status":"false"}
    patient_data['jointpain']={"status":"false"}
    patient_data['hide']=False
    #return 
    return patient_data
