#!/usr/bin/env python
import os
import etcd
import json
import sys

def get_env_variable(var_name):
    """ Get the environment variable or return an exception"""
    try:
        return os.environ[var_name]
    except KeyError:
        error_msg = "Set the %s environment variable" % var_name
        raise ImproperlyConfigured(error_msg)

# If it is already here then just exit()
if os.path.isfile('.project_config.json'):
    print('Project config found, skipping retrieval from etcd.')
    sys.exit()
    
app_name = get_env_variable('APP_NAME').lower()

project_config = open('.project_config.json', 'w')
etcd_host = os.environ.get('ETCD_HOST', '10.30.5.109')

ca = os.path.expanduser('/etcd_certs/ca.crt')
cert = os.path.expanduser('/etcd_certs/res01.crt')
key = os.path.expanduser('/etcd_certs/res01.key')

c = etcd.Client(
    host=etcd_host,
    port=4002,
    ca_cert=ca,
    cert=(cert, key),
    protocol='https'
)

etcd_config = c.read(
    '/applications/{0}/configuration'.format(app_name)
)

project_config.write(etcd_config.value)
project_config.close()